import datetime

from django.contrib.auth.models import User
from django.test import Client
from django.test import TestCase
from django.urls import reverse

from functions import create_subject, create_visit, create_appointment, create_worker, get_test_location
from web.forms import SubjectAddForm, SubjectEditForm
from web.models import Subject
from web.models.constants import SEX_CHOICES_MALE, SUBJECT_TYPE_CHOICES_CONTROL, SUBJECT_TYPE_CHOICES_PATIENT
from web.views.notifications import get_today_midnight_date


class SubjectsViewTests(TestCase):
    def setUp(self):
        self.client = Client()
        username = 'piotr'
        password = 'top_secret'
        self.user = User.objects.create_user(
            username=username, email='jacob@bla', password=password)
        self.worker = create_worker(self.user, True)

        self.client.login(username=username, password=password)

    def test_subjects_add(self):
        self.worker.save()

        response = self.client.get(reverse('web.views.subject_add'))
        self.assertEqual(response.status_code, 200)

    def test_render_subject_edit(self):
        subject = create_subject()

        response = self.client.get(reverse('web.views.subject_edit', kwargs={'id': subject.id}))
        self.assertEqual(response.status_code, 200)

    def test_render_subject_visit_details(self):
        subject = create_subject()
        visit = create_visit(subject)
        create_appointment(visit)

        response = self.client.get(reverse('web.views.subject_visit_details', kwargs={'id': subject.id}))
        self.assertEqual(response.status_code, 200)

    def test_save_subject_edit(self):
        subject = create_subject()
        form_subject = SubjectEditForm(instance=subject)
        form_data = {}
        for key, value in form_subject.initial.items():
            if value is not None:
                form_data[key] = value

        form_data['dead'] = "True"
        form_data['resigned'] = "True"
        response = self.client.post(
            reverse('web.views.subject_edit', kwargs={'id': subject.id}), data=form_data)

        self.assertEqual(response.status_code, 302)
        updated_subject = Subject.objects.filter(id=subject.id)[0]
        self.assertTrue(updated_subject.dead)
        self.assertTrue(updated_subject.resigned)

    def test_subjects_add_2(self):

        form = SubjectAddForm(user=self.user)
        form_data = {}
        for key, value in form.initial.items():
            if value is not None:
                form_data[key] = value

        form_data["country"] = "Luxembourg"
        form_data["first_name"] = "John"
        form_data["last_name"] = "Doe"
        form_data["sex"] = SEX_CHOICES_MALE
        form_data["type"] = SUBJECT_TYPE_CHOICES_CONTROL
        form_data["default_location"] = get_test_location().id
        response = self.client.post(reverse('web.views.subject_add'), data=form_data)
        self.assertEqual(response.status_code, 302)
        response = self.client.get(response.url)
        self.assertContains(response, "Subject created")

        subject = Subject.objects.all()[0]
        self.assertEqual("L-001", subject.screening_number,
                         "prefix should start by L" +
                         " as default location prefix is not defined and subject type is control")

    def test_subjects_add_patient(self):

        form = SubjectAddForm(user=self.user)
        form_data = {}
        for key, value in form.initial.items():
            if value is not None:
                form_data[key] = value

        form_data["country"] = "Luxembourg"
        form_data["first_name"] = "John"
        form_data["last_name"] = "Doe"
        form_data["sex"] = SEX_CHOICES_MALE
        form_data["type"] = SUBJECT_TYPE_CHOICES_PATIENT
        form_data["default_location"] = get_test_location().id
        response = self.client.post(reverse('web.views.subject_add'), data=form_data)
        self.assertEqual(response.status_code, 302)
        response = self.client.get(response.url)
        self.assertContains(response, "Subject created")

        subject = Subject.objects.all()[0]
        self.assertEqual("P-001", subject.screening_number,
                         "prefix should start by P" +
                         " as default location prefix is not defined and subject type is patient")

    def test_subjects_add_with_prefixed_location(self):

        form = SubjectAddForm(user=self.user)
        form_data = {}
        for key, value in form.initial.items():
            if value is not None:
                form_data[key] = value

        form_data["country"] = "Luxembourg"
        form_data["first_name"] = "John"
        form_data["last_name"] = "Doe"
        form_data["sex"] = SEX_CHOICES_MALE
        form_data["type"] = SUBJECT_TYPE_CHOICES_PATIENT
        location = get_test_location()
        location.prefix = 'X'
        location.save()
        form_data["default_location"] = location.id
        response = self.client.post(reverse('web.views.subject_add'), data=form_data)
        self.assertEqual(response.status_code, 302)
        response = self.client.get(response.url)
        self.assertContains(response, "Subject created")

        subject = Subject.objects.all()[0]
        self.assertEqual("X-001", subject.screening_number,
                         "prefix should start by X as default location prefix is equal to 'X'")

    def test_render_subjects(self):
        create_subject()

        response = self.client.get(reverse('web.views.subjects'))
        self.assertEqual(response.status_code, 200)

    def test_render_subjects_with_no_visit(self):
        create_subject()

        response = self.client.get(reverse('web.views.subject_no_visits'))
        self.assertEqual(response.status_code, 200)

    def test_render_subjects_require_contact(self):
        subject = create_subject()
        subject.datetime_contact_reminder = get_today_midnight_date() + datetime.timedelta(days=-1)

        response = self.client.get(reverse('web.views.subject_require_contact'))
        self.assertEqual(response.status_code, 200)
